WORK_FOLDER = f".{__package__}"
DEFAULT_IGNORE = [
    WORK_FOLDER,
    '.git',
    '__pycache__',
    '.idea/',
    'venv',
    '.pytest_cache',
    '.coverage',
    'coverage.xml',
    ".pylintrc",
    '.diff',
    '.patch',
]
